#!/usr/bin/python3
# -*- coding: utf-8 -*-
# (c) 2012 Canonical Ltd.
#
# Authors: Alberto Milone <alberto.milone@canonical.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from Quirks import quirkreader, quirkapplier
# from xkit import xorgparser
# from xkit.xorgparser import *
import sys
import unittest
import os
import logging
import settings


source = settings.inputFile
destination = settings.inputDir
destinationFile = os.path.join(settings.outputDir, 'quirksreader_test.txt')
tempFile = os.path.join(destination, 'tmp')


def get_quirks_from_file(quirk_file):
    '''check all the files in a directory looking for quirks'''
    # read other blacklist files (which we will not touch, but evaluate)
    quirk_file = quirkreader.ReadQuirk(quirk_file)
    return quirk_file.get_quirks()


class QuirkReaderTestCase(unittest.TestCase):

    # def setUp(self):
    #     self.quirk_file = quirkreader.ReadQuirk(quirk_file)
    #     self.quirks = self.quirk_file.get_quirks()

    def tearDown(self):
        # self.parser.comments.insert(0, '\n-----' + self.this_function_name + '-----\n')
        # self.parser.write(destinationFile, test=True)
        try:
            os.remove(tempFile)
        except (OSError, IOError):
            pass

    def test_read_quirk1(self):
        '''1 Matching config file'''
        self.this_function_name = sys._getframe().f_code.co_name

        with open(tempFile, 'w') as confFile:
            confFile.write('''
Section "Quirk"
    Identifier "Test Latitude E6530"
    Handler "nvidia-current|nvidia-current-updates"
    Match "sys_vendor" "Dell Inc."
    Match "product_name" "Latitude E6530"
    XorgSnippet
        Section "Device"
            Identifier "My Card"
            Driver "nvidia"
            Option "NoLogo" "True"
        EndSection

        Section "Screen"
            Identifier "My Screen"
            Option "RegistryDwords" "EnableBrightnessControl=1"
        EndSection
    EndXorgSnippet
EndSection
''')
        # os.system('cat %s' % tempFile)
        # loglevel = logging.DEBUG
    # else:
        # loglevel = logging.INFO

        # logging.basicConfig(format='%(levelname)s:%(message)s', level=loglevel)
        a = quirkapplier.QuirkChecker('nvidia-current', path=destination)

        # Override DMI
        a._system_info = {'sys_vendor': 'Dell Inc.',
                          'bios_vendor': 'American Megatrends Inc.',
                          'product_version': 'System Version',
                          'board_name': 'P6T SE',
                          'bios_date': '01/19/2009',
                          'bios_version': '0106',
                          'product_name': 'Latitude E6530',
                          'board_vendor': 'ASUSTeK Computer INC.'}
        quirk_found = False
        quirk_matches = False

        for quirk in a._quirks:
            if a._handler.lower() in [x.lower().strip() for x in quirk.handler]:
                quirk_found = True

                logging.debug('Processing quirk %s' % quirk.id)
                # self.assertTrue(a.matches_tags(quirk))
                if a.matches_tags(quirk):
                    # Do something here
                    logging.debug('Quirk matches')
                    quirk_matches = True
                else:
                    logging.debug('Quirk doesn\'t match')

        self.assertTrue(quirk_found)
        self.assertTrue(quirk_matches)

    def test_read_quirk2(self):
        '''2 Not matching config file'''
        self.this_function_name = sys._getframe().f_code.co_name

        with open(tempFile, 'w') as confFile:
            confFile.write('''
Section "Quirk"
    Identifier "Test Latitude E6530"
    Handler "nvidia-current|nvidia-current-updates"
    Match "sys_vendor" "Dell Inc."
    Match "product_name" "Latitude E6530"
    XorgSnippet
        Section "Device"
            Identifier "My Card"
            Driver "nvidia"
            Option "NoLogo" "True"
        EndSection

        Section "Screen"
            Identifier "My Screen"
            Option "RegistryDwords" "EnableBrightnessControl=1"
        EndSection
    EndXorgSnippet
EndSection


''')
        # os.system('cat %s' % tempFile)
        # loglevel = logging.DEBUG
    # else:
        # loglevel = logging.INFO

        # logging.basicConfig(format='%(levelname)s:%(message)s', level=loglevel)
        a = quirkapplier.QuirkChecker('nvidia-current', path=destination)

        # Override DMI
        a._system_info = {'sys_vendor': 'Fake',
                          'bios_vendor': 'American Megatrends Inc.',
                          'product_version': 'System Version',
                          'board_name': 'P6T SE',
                          'bios_date': '01/19/2009',
                          'bios_version': '0106',
                          'product_name': 'Fake product',
                          'board_vendor': 'ASUSTeK Computer INC.'}
        quirk_found = False
        quirk_matches = True

        for quirk in a._quirks:
            if a._handler.lower() in [x.lower().strip() for x in quirk.handler]:
                quirk_found = True

                logging.debug('Processing quirk %s' % quirk.id)
                # It doesn't have to match
                self.assertTrue(not a.matches_tags(quirk))
                if a.matches_tags(quirk):
                    # Do something here
                    logging.debug('Quirk matches')
                else:
                    logging.debug('Quirk doesn\'t match')
                    quirk_matches = False

        self.assertTrue(quirk_found)
        self.assertFalse(quirk_matches)

    def test_read_quirk3(self):
        '''3 Matching quirk aimed at multiple products'''
        self.this_function_name = sys._getframe().f_code.co_name

        with open(tempFile, 'w') as confFile:
            confFile.write('''
Section "Quirk"
    Identifier "Test Latitude E6530"
    Handler "nvidia-current|nvidia-current-updates"
    Match "sys_vendor" "Dell Inc."
    Match "product_name" "Latitude E6530|Latitude E6535"
    XorgSnippet
        Section "Device"
            Identifier "My Card"
            Driver "nvidia"
            Option "NoLogo" "True"
        EndSection

        Section "Screen"
            Identifier "My Screen"
            Option "RegistryDwords" "EnableBrightnessControl=1"
        EndSection
    EndXorgSnippet
EndSection


''')
        # os.system('cat %s' % tempFile)
        # loglevel = logging.DEBUG
    # else:
        # loglevel = logging.INFO

        # logging.basicConfig(format='%(levelname)s:%(message)s', level=loglevel)
        a = quirkapplier.QuirkChecker('nvidia-current', path=destination)

        # Override DMI
        a._system_info = {'sys_vendor': 'Dell Inc.',
                          'bios_vendor': 'American Megatrends Inc.',
                          'product_version': 'System Version',
                          'board_name': 'P6T SE',
                          'bios_date': '01/19/2009',
                          'bios_version': '0106',
                          'product_name': 'Latitude E6535',
                          'board_vendor': 'ASUSTeK Computer INC.'}
        quirk_found = False
        quirk_matches = False

        for quirk in a._quirks:
            if a._handler.lower() in [x.lower().strip() for x in quirk.handler]:
                quirk_found = True

                logging.debug('Processing quirk %s' % quirk.id)
                # Let's test only the quirk that matters
                if quirk.id == "Test Latitude E6530":
                    self.assertTrue(a.matches_tags(quirk))
                    if a.matches_tags(quirk):
                        # Do something here
                        logging.debug('Quirk matches')
                        quirk_matches = True
                    else:
                        logging.debug('Quirk doesn\'t match')

        self.assertTrue(quirk_found)
        self.assertTrue(quirk_matches)

    def test_read_quirk4(self):
        '''3 Matching quirk aimed at multiple products only one should match'''
        self.this_function_name = sys._getframe().f_code.co_name

        with open(tempFile, 'w') as confFile:
            confFile.write('''
Section "Quirk"
    Identifier "Test Latitude E6530"
    Handler "nvidia-current|nvidia-current-updates"
    Match "sys_vendor" "Dell Inc."
    Match "product_name" "Latitude E6530|Latitude E6535"
    XorgSnippet
        Section "Device"
            Identifier "My Card"
            Driver "nvidia"
            Option "NoLogo" "True"
        EndSection

        Section "Screen"
            Identifier "My Screen"
            Option "RegistryDwords" "EnableBrightnessControl=1"
        EndSection
    EndXorgSnippet
EndSection


''')
        # os.system('cat %s' % tempFile)
        # loglevel = logging.DEBUG
    # else:
        # loglevel = logging.INFO

        # logging.basicConfig(format='%(levelname)s:%(message)s', level=loglevel)
        a = quirkapplier.QuirkChecker('nvidia-current', path=destination)

        # Override DMI
        a._system_info = {'sys_vendor': 'Dell Inc.',
                          'bios_vendor': 'American Megatrends Inc.',
                          'product_version': 'System Version',
                          'board_name': 'P6T SE',
                          'bios_date': '01/19/2009',
                          'bios_version': '0106',
                          'product_name': 'Latitude E6535',
                          'board_vendor': 'ASUSTeK Computer INC.'}
        quirk_found = False
        quirk_matches = False
        matches_number = 0

        for quirk in a._quirks:
            if a._handler.lower() in [x.lower().strip() for x in quirk.handler]:
                quirk_found = True

                logging.debug('Processing quirk %s' % quirk.id)
                # Let's test only the quirk that matters
                if quirk.id == "Test Latitude E6530":
                    self.assertTrue(a.matches_tags(quirk))
                    if a.matches_tags(quirk):
                        # Do something here
                        logging.debug('Quirk matches')
                        quirk_matches = True
                        matches_number += 1
                    else:
                        logging.debug('Quirk doesn\'t match')

        self.assertTrue(quirk_found)
        self.assertTrue(quirk_matches)
        self.assertTrue(matches_number == 1)


def main():
    return 0


if __name__ == '__main__':
    main()



#
#       kerneldetection.py
#
#       Copyright 2013 Canonical Ltd.
#
#       Author: Alberto Milone <alberto.milone@canonical.com>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import apt_pkg
import logging
import re

from subprocess import Popen


class KernelDetection(object):

    def __init__(self, cache=None):
        if cache:
            self.apt_cache = cache
            self.apt_depcache = apt_pkg.DepCache(cache)
        else:
            apt_pkg.init_config()
            apt_pkg.init_system()
            self.apt_cache = apt_pkg.Cache(None)
            self.apt_depcache = apt_pkg.DepCache(cache)

    def _is_greater_than(self, term1, term2):
        # We don't want to take into account
        # the flavour
        pattern = re.compile('(.+)-([0-9]+)-(.+)')
        match1 = pattern.match(term1)
        match2 = pattern.match(term2)
        if match1:
            term1 = '%s-%s' % (match1.group(1),
                               match1.group(2))
            term2 = '%s-%s' % (match2.group(1),
                               match2.group(2))

        logging.debug('Comparing %s with %s' % (term1, term2))
        command = 'dpkg --compare-versions %s gt %s' % \
                  (term1, term2)
        process = Popen(command.split(' '))
        process.communicate()
        return not process.returncode

    def _get_linux_flavour(self, candidates, image):
        pattern = re.compile(r'linux-image-([0-9]+\.[0-9]+\.[0-9]+)-([0-9]+)-(.+)')
        match = pattern.match(image)
        flavour = ''
        if match:
            flavour = match.group(3)

        return flavour

    def _filter_cache(self, pkg):
        package_name = pkg.name
        if (package_name.startswith('linux-image') and
            'extra' not in package_name and (pkg.current_ver or
                                             self.apt_depcache.marked_install(pkg))):
            return package_name
        else:
            return None

    def _get_linux_metapackage(self, target):
        '''Get the linux headers, linux-image or linux metapackage'''
        metapackage = ''
        image_package = ''
        version = ''
        prefix = 'linux-%s' % ('headers' if target == 'headers' else 'image')

        pattern = re.compile('linux-image-(?:unsigned-)?(.+)-([0-9]+)-(.+)')

        for package_name in map(self._filter_cache, self.apt_cache.packages):
            if package_name:
                match = pattern.match(package_name)
                # Here we filter out packages other than
                # the actual image or header packages
                if match:
                    current_package = match.group(0)
                    current_version = '%s-%s' % (match.group(1),
                                                 match.group(2))
                    # See if the current version is greater than
                    # the greatest that we've found so far
                    if self._is_greater_than(current_version,
                                             version):
                        version = current_version
                        image_package = current_package

        if version:
            if target == 'headers':
                target_package = image_package.replace('image', 'headers')
            else:
                target_package = image_package

            reverse_dependencies = [dep.parent_pkg.name for dep in self.apt_cache[target_package]
                                    .rev_depends_list if dep.parent_pkg.name.startswith(prefix)]

            if reverse_dependencies:
                # This should be something like linux-image-$flavour
                # or linux-headers-$flavour
                metapackage = ''
                for candidate in reverse_dependencies:
                    try:
                        candidate_pkg = self.apt_cache[candidate]
                        if (candidate.startswith(prefix) and (candidate_pkg and
                           (candidate_pkg.current_ver or self.apt_depcache.marked_install(candidate_pkg))) and
                           candidate.replace(prefix, '') > metapackage.replace(prefix, '')):
                            metapackage = candidate
                    except KeyError:
                        continue
                # if we are looking for headers, then we are good
                if target == 'meta':
                    # Let's get the metapackage
                    reverse_dependencies = [dep.parent_pkg.name for dep in self.apt_cache[metapackage]
                                            .rev_depends_list if dep.parent_pkg.name.startswith('linux-')]
                    if reverse_dependencies:
                        flavour = self._get_linux_flavour(reverse_dependencies, target_package)
                        linux_meta = ''
                        for meta in reverse_dependencies:
                            # For example linux-generic-hwe-20.04
                            if meta.startswith('linux-%s-' % (flavour)):
                                linux_meta = meta
                                break
                        # This should be something like linux-$flavour
                        if not linux_meta:
                            # Try the 1st reverse dependency
                            metapackage = reverse_dependencies[0]
                        else:
                            metapackage = linux_meta
        return metapackage

    def get_linux_headers_metapackage(self):
        '''Get the linux headers for the newest_kernel installed'''
        return self._get_linux_metapackage('headers')

    def get_linux_image_metapackage(self):
        '''Get the linux headers for the newest_kernel installed'''
        return self._get_linux_metapackage('image')

    def get_linux_metapackage(self):
        '''Get the linux metapackage for the newest_kernel installed'''
        return self._get_linux_metapackage('meta')

    def get_linux_version(self):
        linux_image_meta = self.get_linux_image_metapackage()
        linux_version = ''
        try:
            # dependencies = self.apt_cache[linux_image_meta].candidate.\
            #                  record['Depends']
            candidate = self.apt_depcache.get_candidate_ver(self.apt_cache[linux_image_meta])
            for dep_list in candidate.depends_list_str.get('Depends'):
                for dep_name, dep_ver, dep_op in dep_list:
                    if dep_name.startswith('linux-image'):
                        linux_version = dep_name.strip().replace('linux-image-', '')
                        break
        except KeyError:
            logging.error('No dependencies can be found for %s' % (linux_image_meta))
            return None

        # if ', ' in dependencies:
        #     deps = dependencies.split(', ')
        #     for dep in deps:
        #         if dep.startswith('linux-image'):
        #             linux_version = dep.replace('linux-image-', '')
        # else:
        #     if dependencies.strip().startswith('linux-image'):
        #         linux_version = dependencies.strip().replace('linux-image-', '')

        return linux_version



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# (c) 2012 Canonical Ltd.
#
# Authors: Alberto Milone <alberto.milone@canonical.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import xkit.xutils
import xkit.xorgparser
import Quirks.quirkinfo

import tempfile
import os

class Quirk:

    def __init__(self, id=None, handler=[], x_snippet="", match_tags={}):
        self.id = id
        self.handler = handler
        self.x_snippet = x_snippet
        self.match_tags = {}.fromkeys(Quirks.quirkinfo.dmi_keys, '')

class ReadQuirk:

    def __init__(self, source=None):
        self.source = source
        
        #See if the source is a file or a file object
        #and act accordingly
        file = self.source
        if file == None:
            lines_list = []
        else:
            if not hasattr(file, 'write'):#it is a file
                myfile = open(file, 'r', encoding='utf-8')
                try:
                    lines_list = myfile.readlines()
                    myfile.close()
                except UnicodeDecodeError:
                    lines_list = []
                myfile.close()
            else:
                #it is a file object
                lines_list = file.readlines()
        
        inside_quirk = False
        has_id = False
        has_handler = False
        inside_x_snippet = False
        self._quirks = []

        it = 0
        for line in lines_list:
            if line.strip().startswith('#'):
                continue
            if inside_quirk:
                if inside_x_snippet:

                    if line.lower().strip().startswith('endxorgsnippet'):
                        inside_x_snippet = False
                        continue
                    else:
                        self._quirks[it].x_snippet += line
                else:
                    #not in x_snippet
                    if not has_id and line.lower().strip().startswith('identifier'):
                        has_id = True
                        temp_str = "identifier"
                        id = line[line.lower().rfind(temp_str) + len(
                             temp_str):].strip().replace('"', '')
                        self._quirks[it].id = id
                        del temp_str
                    elif not has_handler and line.lower().strip().startswith('handler'):
                        has_handler = True
                        temp_str = "handler"
                        handler = line[line.lower().rfind(temp_str) + len(
                             temp_str):].strip().replace('"', '')
                        handlers_list = handler.split('|')
                        self._quirks[it].handler = handlers_list
                        del temp_str
                    elif line.lower().strip().startswith('match'):
                        temp_str = "match"
                        temp_bits = line[line.lower().rfind(temp_str) +
                                    len(temp_str):].strip().split('"')
                        tag_match = ''
                        tag_value = ''
                        tag_values = []
                        for elem in temp_bits:
                            if elem.strip():
                                if not tag_match:
                                    tag_match = elem.strip()
                                    #tag_values = []
                                else:
                                    tag_value = elem.strip()
                                    tag_values = tag_value.split('|')
                                    self._quirks[it].match_tags[tag_match] = tag_values
                                    break
                        del temp_bits
                        del temp_str
                        del tag_values
                    elif line.lower().strip().startswith('xorgsnippet'):
                        inside_x_snippet = True
                        self._quirks[it].x_snippet = ""
                        continue
        
                    elif line.lower().strip().startswith('endsection'):
                        #End Quirk
                        inside_quirk = False
                        if not self._quirks[it].id:
                            self._quirks.pop(it)
                        else:
                            it += 1
            else:
                if line.lower().strip().startswith('section') \
                and "quirk" in line.lower():
                    #Begin Quirk
                    inside_quirk = True
                    temp_quirk = Quirk()
                    self._quirks.append(temp_quirk)
                    del temp_quirk
                    continue

    def get_quirks(self):
        return self._quirks


#if __name__ == "__main__":
    #quirk_file = ReadQuirk("quirk_snippet.txt")
    #quirks = quirk_file.get_quirks()
    #for quirk in quirks:
        #print 'Quirk id: "%s"' % quirk.id
        #for tag in quirk.match_tags.keys():
            #print 'Matching "%s" with value "%s"' % (tag, quirk.match_tags[tag])
        #print quirk.x_snippet

    #tmp_file = tempfile.NamedTemporaryFile(mode='w', delete=False)
    #tmp_file.write(quirk.x_snippet)
    #tmp_file.close()

    #tmp_xkit = xkit.xorgparser.Parser(tmp_file.name)
    #print tmp_xkit.globaldict
    #os.unlink(tmp_file.name)



'''Provide a fake package archive for testing.'''

# (C) 2012 Martin Pitt <martin.pitt@ubuntu.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

import tempfile
import shutil
import os
import subprocess
import atexit


class Archive:
    def __init__(self):
        '''Construct a local package test archive.

        The archive is initially empty. You can create new packages with
        create_deb(). self.path contains the path of the archive, and
        self.apt_source provides an apt source "deb" line.

        It is kept in a temporary directory which gets removed when the Archive
        object gets deleted.
        '''
        self.path = tempfile.mkdtemp()
        self.dist = 'devel'
        self.components = ['main', 'universe', 'restricted', 'multiverse']
        self.archs = ['amd64', 'i386', 'arm64', 'armhf', 'ppc64el', 's390x']

        atexit.register(shutil.rmtree, self.path)

        # Local repository is not signed so trusted is required
        self.apt_source_pattern = 'deb [trusted=yes] file://%s %s %s'
        self.apt_source = self.apt_source_pattern % (
            self.path, self.dist, " ".join(self.components))

        # Configuration file for apt-ftparchive to generate archive indices
        # For simplification Suite and Codename are the same and we nly support
        # one arch. This can be further extended to support other archs if the
        # testsuite requires it
        self.aptftp_conf = os.path.join(self.path, 'aptftp.conf')
        with open(self.aptftp_conf, 'w') as f:
            f.write('''
APT::FTPArchive::Release {{
Origin "Ubuntu";
Label "Ubuntu";
Suite "{suite}";
Codename "{codename}";
Architectures "{archs}";
Components "{components}";
Description "Test Repository";
}};
'''.format(suite=self.dist, codename=self.dist, archs=" ".join(self.archs), components=" ".join(self.components)))

        self.aptftp_generate_conf = os.path.join(self.path, 'aptftpgenerate.conf')
        with open(self.aptftp_generate_conf, 'w') as f:
            f.write('''
Dir::ArchiveDir ".";
Dir::CacheDir ".";
TreeDefault::Directory "pool/$(SECTION)/";
TreeDefault::SrcDirectory "pool/$(SECTION)/";
Default::Packages::Extensions ".deb";
Default::Packages::Compress ". gzip bzip2";
Default::Sources::Compress ". gzip bzip2";
Default::Contents::Compress "gzip bzip2";

Tree "dists/{dist}" {{
  Sections "{components}";
  Architectures "{archs}";
}};
'''.format(dist=self.dist, archs=" ".join(self.archs), components=" ".join(self.components)))

            for arch in self.archs:
                for component in self.components:
                    f.write('''
BinDirectory "dists/{dist}/{component}/binary-{arch}" {{
  Packages "dists/{dist}/{component}/binary-{arch}/Packages";
  Contents "dists/{dist}/Contents-{arch}";
}};
'''.format(dist=self.dist, arch=arch, component=component))

    def create_deb(self, name, version='1', architecture='all',
                   dependencies={}, description='test package', extra_tags={},
                   files={}, component='main', srcpkg=None, update_index=True):
        '''Build a deb package and add it to the archive.

        The only mandatory argument is the package name. You can additionally
        specify the package version (default '1'), architecture (default
        'all'), a dictionary with dependencies (empty by default; for example
        {'Depends': 'foo, bar', 'Conflicts: baz'}, a short description
        (default: 'test package'), and arbitrary extra tags.

        By default the package is empty. It can get files by specifying a
        path -> contents dictionary in 'files'. Paths must be relative.
        Example: files={'etc/foo.conf': 'enable=true'}

        The newly created deb automatically gets added to the "Packages" index,
        unless update_index is False.

        Return the path to the newly created deb package, in case you only need
        the deb itself, not the archive.
        '''
        d = tempfile.mkdtemp()
        os.mkdir(os.path.join(d, 'DEBIAN'))
        with open(os.path.join(d, 'DEBIAN', 'control'), 'w') as f:
            f.write('''Package: %s
Maintainer: Test User <test@example.com>
Version: %s
Priority: optional
Section: devel
Architecture: %s
''' % (name, version, architecture))

            for k, v in dependencies.items():
                f.write('%s: %s\n' % (k, v))

            f.write('''Description: %s
 Test dummy package.
''' % description)

            for k, v in extra_tags.items():
                f.write('%s: %s\n' % (k, v))

        for path, contents in files.items():
            if isinstance(contents, bytes):
                mode = 'wb'
            else:
                mode = 'w'
            pathdir = os.path.join(d, os.path.dirname(path))
            if not os.path.isdir(pathdir):
                os.makedirs(pathdir)
            with open(os.path.join(d, path), mode) as f:
                f.write(contents)

        if srcpkg is None:
            srcpkg = name
        if srcpkg.startswith('lib'):
            prefix = srcpkg[:4]
        else:
            prefix = srcpkg[0]
        dir = os.path.join(self.path, 'pool', component, prefix, srcpkg)
        if not os.path.isdir(dir):
            os.makedirs(dir)

        debpath = os.path.join(dir, '%s_%s_%s.deb' % (name, version,
                                                      architecture))
        subprocess.check_call(['dpkg', '-b', d, debpath],
                              stdout=subprocess.PIPE, stderr=subprocess.PIPE)

        shutil.rmtree(d)
        assert os.path.exists(debpath)

        if update_index:
            self.update_index()

        return debpath

    def update_index(self):
        '''Update the "Packages" index.

        This usually gets done automatically by create_deb(), but needs to be
        done if you manually copy debs into the archive or call create_deb with
        update_index==False.
        '''
        old_cwd = os.getcwd()
        try:
            os.chdir(self.path)
            devnull = open(os.devnull, 'w')     # Make apt-ftparchive quiet
            dists_dir = os.path.join(self.path, 'dists')

            # Completely recreates the dist directory to ensure there is no
            # leftover from a previous test.
            if os.path.isdir(dists_dir):
                shutil.rmtree(dists_dir)
            for arch in self.archs:
                for component in self.components:
                    os.makedirs(os.path.join(dists_dir, self.dist, component, 'binary-%s' % arch))

            subprocess.check_call(['apt-ftparchive', 'generate', '-c',
                                   self.aptftp_conf,
                                   self.aptftp_generate_conf],
                                  stderr=devnull)

            with open(os.path.join(dists_dir, self.dist, 'Release'), 'w') as f:
                subprocess.check_call(['apt-ftparchive', 'release', '-c',
                                       self.aptftp_conf,
                                       os.path.join(dists_dir, self.dist)],
                                      stdout=f, stderr=devnull)

            # This is still required by the aptdaemon test structure
            with open('Packages', 'w') as f:
                subprocess.check_call(['apt-ftparchive', 'packages', '.'],
                                      stdout=f, stderr=devnull)

        finally:
            os.chdir(old_cwd)
            devnull.close()

# a = Archive()
# a.create_deb('vanilla')
# a.create_deb('chocolate', dependencies={'Depends': 'foo'},
#     extra_tags={'Modaliases': 'pci-1'},
#     files={'usr/share/doc/chocolate/README': 'hello'})
# print(a.apt_source)
# subprocess.call(['bash', '-i'], cwd=a.path)



# ubuntu-drivers-common custom detect plugin for sl-modem
#
# (C) 2012 Canonical Ltd.
# Author: Martin Pitt <martin.pitt@ubuntu.com>

import os
import re
import logging
import subprocess

modem_re = re.compile(r'^\s*\d+\s*\[Modem\s*\]')
modem_as_subdevice_re = re.compile(r'^card [0-9].*[mM]odem')

pkg = 'sl-modem-daemon'


def detect(apt_cache):
    # Check in /proc/asound/cards
    try:
        with open('/proc/asound/cards') as fd:
            for line in fd:
                if modem_re.match(line):
                    return [pkg]
    except IOError as e:
        logging.debug('could not open /proc/asound/cards: %s', e)

    # Check aplay -l
    try:
        env = os.environ.copy()
        try:
            del env['LANGUAGE']
        except KeyError:
            pass
        env['LC_ALL'] = 'C'
        try:
            aplay = subprocess.Popen(
                ['aplay', '-l'], env=env,
                stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                universal_newlines=True)
            (aplay_out, aplay_err) = aplay.communicate()
            if aplay.returncode != 0:
                logging.error('aplay -l failed with %i: %s' % (aplay.returncode,
                              aplay_err))
                return None
        except FileNotFoundError:
            logging.error('aplay command not found')
            return None

    except OSError:
        logging.debug('could not open aplay -l. Skipping sl-modem detection')
        return None

    for row in aplay_out.splitlines():
        if modem_as_subdevice_re.match(row):
            return [pkg]

    return None
